@@ -168,7 +168,7 @@ cleandirs( char *path, struct llist *khead )
 	 */
 	if ( strcmp( fsitem, base_kfile ) == 0 ||
 		( strncmp( kdir, path, strlen( path )) == 0
-		&& strcmp( base_kfile, de->d_name )) == 0 ) {
+		&& strcmp( base_kfile, de->d_name ) == 0 )) {
 	    continue;
 	}
 
@@ -546,10 +546,11 @@ check( SNET *sn, char *type, char *file )
     int
 main( int argc, char **argv )
 {
-    int			c, port = htons( 6662 ), err = 0;
+    int			c, err = 0;
     int			authlevel = _RADMIND_AUTHLEVEL;
     int			use_randfile = 0;
     int			clean = 0;
+    unsigned short	port = 0;
     char	lcksum[ SZ_BASE64_E( EVP_MAX_MD_SIZE ) ];
     char	tcksum[ SZ_BASE64_E( EVP_MAX_MD_SIZE ) ];
     struct stat		tst, lst;
@@ -557,7 +558,6 @@ main( int argc, char **argv )
     char		*host = _RADMIND_HOST, *p;
     char		path[ MAXPATHLEN ];
     char		tempfile[ MAXPATHLEN ];
-    struct servent	*se;
     char	        **capa = NULL;		/* capabilities */
 
     while (( c = getopt( argc, argv,
@@ -598,13 +598,8 @@ main( int argc, char **argv )
 	    break;
 
 	case 'p':
-	    if (( port = htons ( atoi( optarg )) ) == 0 ) {
-		if (( se = getservbyname( optarg, "tcp" )) == NULL ) {
-		    fprintf( stderr, "%s: service unknown\n", optarg );
-		    exit( 2 );
-		}
-		port = se->s_port;
-	    }
+	    /* connect.c handles things if atoi returns 0 */
+	    port = htons( atoi( optarg )); 
 	    break;
 	
         case 'P' :              /* ca dir */
@@ -1021,6 +1016,10 @@ read_kfile( char * kfile )
 		exit( 2 );
 	    }
 	    break;
+
+	case 'x':
+	    /* exclude patterns have no associated transcript */
+	    break;
 	}
     }
     if ( ferror( f )) {
