@@ -230,11 +230,11 @@ do_line( char *tline, int present, struct stat *st, SNET *sn )
     int
 main( int argc, char **argv )
 {
-    int			c, port = htons( 6662 ), err = 0;
+    int			c, err = 0;
+    unsigned short	port = 0;
     extern int          optind;
     FILE		*f = NULL; 
     char		*host = _RADMIND_HOST, *d_path;
-    struct servent	*se;
     char		tline[ 2 * MAXPATHLEN ];
     char		targvline[ 2 * MAXPATHLEN ];
     char		path[ 2 * MAXPATHLEN ];
@@ -294,13 +294,8 @@ main( int argc, char **argv )
 	    break;
 
 	case 'p':
-	    if (( port = htons ( atoi( optarg ))) == 0 ) {
-		if (( se = getservbyname( optarg, "tcp" )) == NULL ) {
-		    fprintf( stderr, "%s: service unknown\n", optarg );
-		    exit( 2 );
-		}
-		port = se->s_port;
-	    }
+	    /* connect.c handles things if atoi returns 0 */
+	    port = htons( atoi( optarg ));
 	    break;
 
         case 'P' :              /* ca dir */
